//
//  TonePlayer.m
//  iTone
//
//  Created by Allen Porter on 4/20/08.
//  Copyright 2008 __MyCompanyName__. All rights reserved.
//

#import "TonePlayer.h"
#import <ScreenSaver/ScreenSaver.h>

#define PI 3.14159265

AudioDeviceID GetDefaultOutputDevice() {
    AudioDeviceID device;
    UInt32 size = sizeof(AudioDeviceID);
    OSStatus ret;
    ret = AudioHardwareGetProperty(kAudioHardwarePropertyDefaultOutputDevice,
                                   &size,
                                   &device);
    assert(ret == 0);
    return device;
}

AudioStreamID GetStreamForDevice(AudioDeviceID device, bool input) {
    OSStatus ret;
    UInt32 size;
    ret = AudioDeviceGetPropertyInfo(device,
                                     0,
                                     input,
                                     kAudioDevicePropertyStreams,
                                     &size,
                                     NULL);
    assert(ret == 0);
    assert(size > 0);
    int num_streams = size / sizeof(AudioStreamID);
    AudioStreamID* streams = (AudioStreamID*)malloc(num_streams * sizeof(AudioStreamID));
    ret = AudioDeviceGetProperty(device,
                                 0,
                                 input,
                                 kAudioDevicePropertyStreams,
                                 &size,
                                 streams);
    AudioStreamID stream = streams[0];
    free(streams);
    return stream;
}

AudioStreamID GetDefaultOutputStream() {
    return GetStreamForDevice(GetDefaultOutputDevice(), false);
}

bool GetStreamDescription(AudioStreamID stream_id,
                          AudioStreamBasicDescription* description) {
    OSStatus ret;
    UInt32 size = sizeof(AudioStreamBasicDescription);;
    ret = AudioStreamGetProperty(stream_id,
                                 0,
                                 kAudioStreamPropertyPhysicalFormat,
                                 &size,
                                 description);
    return (ret == 0);
}

OSStatus Proc(AudioDeviceID inDevice,
              const AudioTimeStamp* inNow,
              const AudioBufferList* inInputData,
              const AudioTimeStamp* inInputTime,
              AudioBufferList* outOutputData,
              const AudioTimeStamp* inOutputTime,
              void* inClientData,
			  float* myAudioArray)
			  {
    assert(outOutputData->mNumberBuffers > 0);
    assert(inOutputTime->mSampleTime != 0);
    
    TonePlayer* state = (TonePlayer*)inClientData;
    
    AudioBuffer* buf = &outOutputData->mBuffers[0];
    int units = buf->mDataByteSize / sizeof(float);
    float* data = (float*)buf->mData;
    float m = 2 / state->description.mSampleRate;
    for (int i = 0; i < units; ++i) {
        [state setSample:(([state sample] + 1) % (int)state->description.mSampleRate)];
        int t = [state sample];
        if ([state mode] == SAWTOOTH) {
				// do nothing at the moment
        } else {
            
		
//			data[i] = myAudioArray[i] ;					// copy from float array to buffer / not working at the moment
			
//			data[i] = SSRandomFloatBetween(0.0, 1.0);     // generates noise
			
			
			data[i] = sinf((2 * PI * [state frequency] * t) / state->description.mSampleRate);  // Generates the tone
			
			
			
			
//			data[i] = (SSRandomFloatBetween(0.0, 1.0) )/ state->description.mSampleRate);
/*            switch ([state mode]) {
                case SQUARE:
                    if (data[i] > 0) {
                        data[i] = 1;
                    } else if (data[i] < 0) {
                        data[i] = -1;
                    } 
                    break;
                case THEREMIN:
                    if (data[i] < 0) {
                        data[i] = 0 - data[i];
                    }
                   break;
*/             }
     //   }
    }
    return 0;
}

@implementation TonePlayer

@synthesize frequency;
@synthesize sample;
@synthesize mode;
@synthesize audioArray; // added by franz to receive float array as audio buffer

- (id) init
{
    self = [super init];
    if (self != nil) {
        sample = 0;
        mode = SIN;
        frequency = 500;
        device = GetDefaultOutputDevice();
		
        if (!GetStreamDescription(GetDefaultOutputStream(), &description)) {
            return nil;
        }
        OSStatus status = AudioDeviceAddIOProc(device, &Proc, self);
        if (status) {
            NSLog(@"AudioDeviceAddIOProc failed: %lld (%s)");
            return nil;
        }
        
    }
    return self;
}

- (BOOL) start
{
    NSLog(@"Starting w/ Frequency=%d", frequency);
    OSStatus status = AudioDeviceStart(device, &Proc);
    if (status) {
        NSLog(@"AudioDeviceStart failed: %lld", status);
        return NO;
    }
    return YES;
}

- (BOOL) stop
{
    OSStatus status = AudioDeviceStop(device, &Proc);
    if (status) {
        NSLog(@"AudioDeviceStop failed: %lld (%s)", status);
        return NO;
    }
    return YES;
}

@end
